@@ -48,8 +48,8 @@ N_ = gpodder.ngettext
 
 
 def parse_app_menu_for_accels(filename):
-    """
-    grab (accelerator, action) bindings from menus.ui.
+    """Grab (accelerator, action) bindings from menus.ui.
+
     See #815 Ctrl-Q doesn't quit for justification.
     Unfortunately it's not available from the Gio.MenuModel we get from the Gtk.Builder,
     so we get it ourself.
@@ -95,6 +95,10 @@ class gPodderApplication(Gtk.Application):
         action.connect('activate', self.on_help_activate)
         self.add_action(action)
 
+        action = Gio.SimpleAction.new('logs', None)
+        action.connect('activate', self.on_logs_activate)
+        self.add_action(action)
+
         action = Gio.SimpleAction.new('preferences', None)
         action.connect('activate', self.on_itemPreferences_activate)
         self.add_action(action)
@@ -118,6 +122,7 @@ class gPodderApplication(Gtk.Application):
 
         builder = Gtk.Builder()
         builder.set_translation_domain(gpodder.textdomain)
+        self.builder = builder
 
         menu_filename = None
         for ui_folder in gpodder.ui_folders:
@@ -132,6 +137,7 @@ class gPodderApplication(Gtk.Application):
             logger.error('Cannot find gtk/menus.ui in %r, exiting' % gpodder.ui_folders)
             sys.exit(1)
 
+        self.menu_extras = builder.get_object('menuExtras')
         self.menu_view_columns = builder.get_object('menuViewColumns')
         self.set_menubar(menubar)
 
@@ -171,7 +177,7 @@ class gPodderApplication(Gtk.Application):
 
             self.bus_name = dbus.service.BusName(gpodder.dbus_bus_name, bus=gpodder.dbus_session_bus)
         except dbus.exceptions.DBusException as dbe:
-            logger.warn('Cannot get "on the bus".', exc_info=True)
+            logger.warning('Cannot get "on the bus".', exc_info=True)
             dlg = Gtk.MessageDialog(None, Gtk.DialogFlags.MODAL, Gtk.MessageType.ERROR,
                    Gtk.ButtonsType.CLOSE, _('Cannot start gPodder'))
             dlg.format_secondary_markup(_('D-Bus error: %s') % (str(dbe),))
@@ -194,21 +200,81 @@ class gPodderApplication(Gtk.Application):
                 # Handle "subscribe to podcast" events from firefox
                 macosx.register_handlers(self.window)
 
+            # Set dark mode from color_scheme config key, or from Settings portal
+            # if it exists and color_scheme is 'system'.
+            if getattr(gpodder.dbus_session_bus, 'fake', False):
+                self.have_settings_portal = False
+                self._set_default_color_scheme('light')
+                self.set_dark_mode(self.window.config.ui.gtk.color_scheme == 'dark')
+            else:
+                self.read_portal_color_scheme()
+                gpodder.dbus_session_bus.add_signal_receiver(
+                    self.on_portal_setting_changed, "SettingChanged", None,
+                    "org.freedesktop.portal.Desktop", "/org/freedesktop/portal/desktop")
+
         self.window.gPodder.present()
 
+    def _set_default_color_scheme(self, default):
+        """Set the default value for color_scheme based on GTK settings.
+
+        If gtk_application_prefer_dark_theme is set to 1 (a non-default value),
+        the user has set it in GTK settings.ini and we set color_scheme to match
+        this preference. Otherwise we set the key to the given default, which
+        should be 'system' in case Settings portal is found, or 'light' if it's not.
+        """
+        if self.window.config.ui.gtk.color_scheme is None:
+            settings = Gtk.Settings.get_default()
+            self.window.config.ui.gtk.color_scheme = (
+                'dark' if settings.props.gtk_application_prefer_dark_theme == 1
+                else default)
+
+    def set_dark_mode(self, dark):
+        settings = Gtk.Settings.get_default()
+        settings.props.gtk_application_prefer_dark_theme = 1 if dark else 0
+
+    def read_portal_color_scheme(self):
+        gpodder.dbus_session_bus.call_async(
+            "org.freedesktop.portal.Desktop", "/org/freedesktop/portal/desktop",
+            "org.freedesktop.portal.Settings", "ReadOne", "ss",
+            ("org.freedesktop.appearance", "color-scheme"),
+            self.on_portal_settings_read, self.on_portal_settings_read_error)
+
+    def on_portal_settings_read(self, value):
+        self.have_settings_portal = True
+        self._set_default_color_scheme('system')
+        if self.window.config.ui.gtk.color_scheme == 'system':
+            self.set_dark_mode(value == 1)
+        else:
+            self.set_dark_mode(self.window.config.ui.gtk.color_scheme == 'dark')
+
+    def on_portal_settings_read_error(self, value):
+        self.have_settings_portal = False
+        self._set_default_color_scheme('light')
+        self.set_dark_mode(self.window.config.ui.gtk.color_scheme == 'dark')
+
+    def on_portal_setting_changed(self, namespace, key, value):
+        if (namespace == 'org.freedesktop.appearance'
+                and key == 'color-scheme'):
+            dark = (value == 1)
+            if self.window.config.ui.gtk.color_scheme == 'system':
+                logger.debug(
+                    f"'color-scheme' changed to {value}, setting dark mode to {dark}")
+                self.set_dark_mode(dark)
+
     def on_menu(self, action, param):
         self.menu_popover.popup()
 
     def on_about(self, action, param):
         dlg = Gtk.Dialog(_('About gPodder'), self.window.gPodder,
                 Gtk.DialogFlags.MODAL)
-        dlg.add_button(Gtk.STOCK_CLOSE, Gtk.ResponseType.OK).show()
+        dlg.add_button(_('_Close'), Gtk.ResponseType.OK).show()
         dlg.set_resizable(True)
 
         bg = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6, margin=16)
         pb = GdkPixbuf.Pixbuf.new_from_file_at_size(gpodder.icon_file, 160, 160)
         bg.pack_start(Gtk.Image.new_from_pixbuf(pb), False, False, 0)
         label = Gtk.Label(justify=Gtk.Justification.CENTER)
+        label.set_selectable(True)
         label.set_markup('\n'.join(x.strip() for x in """
         <b>gPodder {version} ({date})</b>
 
@@ -225,6 +291,7 @@ class gPodderApplication(Gtk.Application):
                    url=html.escape(gpodder.__url__),
                    tr_website=_('Website'),
                    tr_bugtracker=_('Bug Tracker')).strip().split('\n')))
+        label.connect('activate-link', lambda label, url: util.open_website(url))
 
         bg.pack_start(label, False, False, 0)
         bg.pack_start(Gtk.Label(), False, False, 0)
@@ -245,6 +312,9 @@ class gPodderApplication(Gtk.Application):
     def on_help_activate(self, action, param):
         util.open_website('https://gpodder.github.io/docs/')
 
+    def on_logs_activate(self, action, param):
+        util.gui_open(os.path.join(gpodder.home, 'Logs'), gui=self.window)
+
     def on_itemPreferences_activate(self, action, param=None):
         gPodderPreferences(self.window.gPodder,
                 _config=self.window.config,
@@ -254,13 +324,17 @@ class gPodderApplication(Gtk.Application):
                 on_send_full_subscriptions=self.window.on_send_full_subscriptions,
                 on_itemExportChannels_activate=self.window.on_itemExportChannels_activate,
                 on_extension_enabled=self.on_extension_enabled,
-                on_extension_disabled=self.on_extension_disabled)
+                on_extension_disabled=self.on_extension_disabled,
+                have_settings_portal=self.have_settings_portal)
 
     def on_goto_mygpo(self, action, param):
         self.window.mygpo_client.open_website()
 
     def on_check_for_updates_activate(self, action, param):
-        self.window.check_for_updates(silent=False)
+        if os.path.exists(gpodder.no_update_check_file):
+            self.window.check_for_distro_updates()
+        else:
+            self.window.check_for_updates(silent=False)
 
     def on_extension_enabled(self, extension):
         self.window.on_extension_enabled(extension)
