@@ -19,6 +19,7 @@
 
 import html
 import logging
+from urllib.request import getproxies
 
 from gi.repository import Gdk, Gtk, Pango
 
@@ -119,8 +120,8 @@ class YouTubeVideoFormatListModel(Gtk.ListStore):
             }
             self.append((caption, 0))
 
-        for id, (fmt_id, path, description) in youtube.formats:
-            self.append((description, id))
+        for fmt, (fmt_id, path, description) in youtube.formats:
+            self.append((description, fmt))
 
     def get_index(self):
         for index, row in enumerate(self):
@@ -145,8 +146,8 @@ class YouTubeVideoHLSFormatListModel(Gtk.ListStore):
             }
             self.append((caption, 0))
 
-        for id, (fmt_id, path, description) in youtube.hls_formats:
-            self.append((description, id))
+        for fmt, (fmt_id, path, description) in youtube.hls_formats:
+            self.append((description, fmt))
 
     def get_index(self):
         for index, row in enumerate(self):
@@ -180,6 +181,26 @@ class VimeoVideoFormatListModel(Gtk.ListStore):
             self._config.vimeo.fileformat = value
 
 
+class ProxyTypeActionList(Gtk.ListStore):
+    C_CAPTION, C_PROXY_TYPE = list(range(2))
+
+    def __init__(self, config):
+        Gtk.ListStore.__init__(self, str, str)
+        self._config = config
+        self.append((_('SOCKS5h (Remote DNS)'), 'socks5h'))
+        self.append((_('SOCKS5'), 'socks5'))
+        self.append((_('HTTP'), 'http'))
+
+    def get_index(self):
+        for index, row in enumerate(self):
+            if self._config.network.proxy_type == row[self.C_PROXY_TYPE]:
+                return index
+        return 0
+
+    def set_index(self, index):
+        self._config.network.proxy_type = self[index][self.C_PROXY_TYPE]
+
+
 class gPodderPreferences(BuilderWidget):
     C_TOGGLE, C_LABEL, C_EXTENSION, C_SHOW_TOGGLE = list(range(4))
 
@@ -205,6 +226,21 @@ class gPodderPreferences(BuilderWidget):
         index = self.video_player_model.get_index(self._config.player.video)
         self.combo_video_player_app.set_active(index)
 
+        self.combo_color_scheme.remove_all()
+        self.combo_color_scheme.prepend('dark', 'Dark')
+        self.combo_color_scheme.prepend('light', 'Light')
+        cs = self._config.ui.gtk.color_scheme
+        if self.have_settings_portal:
+            self.combo_color_scheme.prepend('system', 'System')
+            self.combo_color_scheme.set_active_id(cs)
+        else:
+            if cs == 'system':
+                self.combo_color_scheme.set_active_id('light')
+                self._config.ui.gtk.color_scheme = 'light'
+            else:
+                self.combo_color_scheme.set_active_id(cs)
+        self._config.connect_gtk_combo_box_text('ui.gtk.color_scheme', self.combo_color_scheme)
+
         self.preferred_youtube_format_model = YouTubeVideoFormatListModel(self._config)
         self.combobox_preferred_youtube_format.set_model(self.preferred_youtube_format_model)
         cellrenderer = Gtk.CellRendererText()
@@ -232,6 +268,20 @@ class gPodderPreferences(BuilderWidget):
         self._config.connect_gtk_togglebutton('ui.gtk.find_as_you_type',
                                               self.checkbutton_find_as_you_type)
 
+        self._config.connect_gtk_togglebutton('ui.gtk.podcast_list.hide_empty',
+                                              self.checkbutton_podcast_list_hide_empty)
+        self._config.connect_gtk_togglebutton('ui.gtk.podcast_list.all_episodes',
+                                              self.checkbutton_podcast_list_all_episodes)
+        self._config.connect_gtk_togglebutton('ui.gtk.podcast_list.sections',
+                                              self.checkbutton_podcast_list_sections)
+
+        self._config.connect_gtk_togglebutton('ui.gtk.episode_list.always_show_new',
+                                              self.checkbutton_episode_list_always_show_new)
+        self._config.connect_gtk_togglebutton('ui.gtk.episode_list.trim_title_prefix',
+                                              self.checkbutton_episode_list_trim_title_prefix)
+        self._config.connect_gtk_togglebutton('ui.gtk.episode_list.descriptions',
+                                              self.checkbutton_episode_list_descriptions)
+
         self.update_interval_presets = [0, 10, 30, 60, 2 * 60, 6 * 60, 12 * 60]
         adjustment_update_interval = self.hscale_update_interval.get_adjustment()
         adjustment_update_interval.set_upper(len(self.update_interval_presets) - 1)
@@ -316,6 +366,32 @@ class gPodderPreferences(BuilderWidget):
         # Disable mygpo sync while the dialog is open
         self._config.mygpo.enabled = False
 
+        # Network proxy settings UI
+        self._config.connect_gtk_togglebutton('network.use_proxy',
+                                              self.checkbutton_use_proxy)
+        self._config.connect_gtk_togglebutton('network.proxy_use_username_password',
+                                              self.checkbutton_proxy_use_username_password)
+        self.entry_proxy_hostname.set_text(self._config.network.proxy_hostname)
+        self.entry_proxy_port.set_text(self._config.network.proxy_port)
+        # This will disable the proxy input details on creation if checkbutton
+        # is unticked (value from _config) on each preferences menu creation
+        self.on_checkbutton_use_proxy_toggled(self.checkbutton_use_proxy)
+        self.on_checkbutton_proxy_use_username_password_toggled(self.checkbutton_proxy_use_username_password)
+        self.proxy_type_model = ProxyTypeActionList(self._config)
+        self.combobox_proxy_type.set_model(self.proxy_type_model)
+        self.combobox_proxy_type.pack_start(cellrenderer, True)
+        self.combobox_proxy_type.add_attribute(cellrenderer, 'text',
+                                               ProxyTypeActionList.C_CAPTION)
+        self.combobox_proxy_type.set_active(self.proxy_type_model.get_index())
+        env_proxies = getproxies()
+        self.label_env_proxy_descr.set_visible(bool(env_proxies))
+        self.label_env_proxy.set_visible(bool(env_proxies))
+        if env_proxies:
+            env_proxies_str = ''
+            for var, url in env_proxies.items():
+                env_proxies_str += f"{var}_proxy={url}\n"
+            self.label_env_proxy.set_text(env_proxies_str)
+
         # Configure the extensions manager GUI
         self.set_extension_preferences()
 
@@ -574,8 +650,8 @@ class gPodderPreferences(BuilderWidget):
         ret = ret.replace(' ', '\xa0')
         return ret
 
-    def on_update_interval_value_changed(self, range):
-        value = int(range.get_value())
+    def on_update_interval_value_changed(self, gtk_range):
+        value = int(gtk_range.get_value())
         self._config.auto.update.enabled = (value > 0)
         self._config.auto.update.frequency = self.update_interval_presets[value]
 
@@ -591,8 +667,8 @@ class gPodderPreferences(BuilderWidget):
             return N_('after %(count)d day', 'after %(count)d days',
                       value) % {'count': value}
 
-    def on_expiration_value_changed(self, range):
-        value = int(range.get_value())
+    def on_expiration_value_changed(self, gtk_range):
+        value = int(gtk_range.get_value())
 
         if value == 0:
             self.checkbutton_expiration_unplayed.set_active(False)
@@ -745,3 +821,28 @@ class gPodderPreferences(BuilderWidget):
             break
 
         fs.destroy()
+
+    def on_checkbutton_use_proxy_toggled(self, widget):
+        widgets = (self.grid_network_proxy_details,
+                   self.vbox_network_proxy_username_password)
+        for w in widgets:
+            w.set_sensitive(widget.get_active())
+
+    def on_checkbutton_proxy_use_username_password_toggled(self, widget):
+        self.grid_network_proxy_username_password.set_sensitive(widget.get_active())
+
+    def on_combobox_proxy_type_changed(self, widget):
+        index = self.combobox_proxy_type.get_active()
+        self.proxy_type_model.set_index(index)
+
+    def on_proxy_hostname_changed(self, widget):
+        self._config.network.proxy_hostname = widget.get_text()
+
+    def on_proxy_port_changed(self, widget):
+        self._config.network.proxy_port = widget.get_text()
+
+    def on_proxy_username_changed(self, widget):
+        self._config.network.proxy_username = widget.get_text()
+
+    def on_proxy_password_changed(self, widget):
+        self._config.network.proxy_password = widget.get_text()
