@@ -17,7 +17,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-import cgi
+import html
 
 from gi.repository import Gtk
 
@@ -30,6 +30,7 @@ _ = gpodder.gettext
 
 class gPodderConfigEditor(BuilderWidget):
     def new(self):
+        self.gPodderConfigEditor.set_transient_for(self.parent_widget)
         name_column = Gtk.TreeViewColumn(_('Setting'))
         name_renderer = Gtk.CellRendererText()
         name_column.pack_start(name_renderer, True)
@@ -40,6 +41,8 @@ class gPodderConfigEditor(BuilderWidget):
 
         value_column = Gtk.TreeViewColumn(_('Set to'))
         value_check_renderer = Gtk.CellRendererToggle()
+        # align left otherwise the checkbox is very far away and not visible
+        value_check_renderer.set_alignment(0, 0.5)
         value_column.pack_start(value_check_renderer, False)
         value_column.add_attribute(value_check_renderer, 'active', 7)
         value_column.add_attribute(value_check_renderer, 'visible', 6)
@@ -63,33 +66,35 @@ class gPodderConfigEditor(BuilderWidget):
         self.configeditor.set_model(self.filter)
         self.configeditor.set_rules_hint(True)
 
-    def visible_func(self, model, iter, user_data=None):
+        self._config.connect_gtk_window(self.main_window, 'config_editor', True)
+
+    def visible_func(self, model, iterator, user_data=None):
         text = self.entryFilter.get_text().lower()
         if text == '':
             return True
         else:
             # either the variable name or its value
-            return (text in model.get_value(iter, 0).lower() or
-                    text in model.get_value(iter, 2).lower())
+            return (text in model.get_value(iterator, 0).lower()
+                    or text in model.get_value(iterator, 2).lower())
 
     def value_edited(self, renderer, path, new_text):
         model = self.configeditor.get_model()
-        iter = model.get_iter(path)
-        name = model.get_value(iter, 0)
-        type_cute = model.get_value(iter, 1)
+        iterator = model.get_iter(path)
+        name = model.get_value(iterator, 0)
+        type_cute = model.get_value(iterator, 1)
 
         if not self._config.update_field(name, new_text):
             message = _('Cannot set %(field)s to %(value)s. Needed data type: %(datatype)s')
-            d = {'field': cgi.escape(name),
-                 'value': cgi.escape(new_text),
-                 'datatype': cgi.escape(type_cute)}
+            d = {'field': html.escape(name),
+                 'value': html.escape(new_text),
+                 'datatype': html.escape(type_cute)}
             self.notification(message % d, _('Error setting option'))
 
     def value_toggled(self, renderer, path):
         model = self.configeditor.get_model()
-        iter = model.get_iter(path)
-        field_name = model.get_value(iter, 0)
-        field_type = model.get_value(iter, 3)
+        iterator = model.get_iter(path)
+        field_name = model.get_value(iterator, 0)
+        field_type = model.get_value(iterator, 3)
 
         # Flip the boolean config flag
         if field_type == bool:
