@@ -41,15 +41,6 @@ _ = gpodder.gettext
 logger = logging.getLogger(__name__)
 
 
-try:
-    from gi.repository import Gio
-    have_gio = True
-except ImportError:
-    have_gio = False
-
-# ----------------------------------------------------------
-
-
 class GEpisode(model.PodcastEpisode):
     __slots__ = ()
 
@@ -98,7 +89,7 @@ class GPodcast(model.PodcastChannel):
 
     @property
     def title_markup(self):
-        """ escaped title for the mass unsubscribe dialog """
+        """Escaped title for the mass unsubscribe dialog."""
         return html.escape(self.title)
 
 
@@ -109,7 +100,8 @@ class Model(model.Model):
 
 
 # Singleton indicator if a row is a section
-class SeparatorMarker(object): pass
+class SeparatorMarker(object):
+    pass
 
 
 class BackgroundUpdate(object):
@@ -128,7 +120,7 @@ class BackgroundUpdate(object):
                 model.C_URL, episode.url,
                 model.C_TITLE, episode.title,
                 model.C_EPISODE, episode,
-                model.C_PUBLISHED_TEXT, episode.cute_pubdate(),
+                model.C_PUBLISHED_TEXT, episode.cute_pubdate(show_time=self.model._config_ui_gtk_episode_list_show_released_time),
                 model.C_PUBLISHED, episode.published,
             )
             update_fields = model.get_update_fields(episode)
@@ -215,11 +207,13 @@ class EpisodeListModel(Gtk.ListStore):
         self._config_ui_gtk_episode_list_always_show_new = False
         self._config_ui_gtk_episode_list_trim_title_prefix = False
         self._config_ui_gtk_episode_list_descriptions = False
+        self._config_ui_gtk_episode_list_show_released_time = False
 
     def cache_config(self, config):
         self._config_ui_gtk_episode_list_always_show_new = config.ui.gtk.episode_list.always_show_new
         self._config_ui_gtk_episode_list_trim_title_prefix = config.ui.gtk.episode_list.trim_title_prefix
         self._config_ui_gtk_episode_list_descriptions = config.ui.gtk.episode_list.descriptions
+        self._config_ui_gtk_episode_list_show_released_time = config.ui.gtk.episode_list.show_released_time
 
     def _format_filesize(self, episode):
         if episode.file_size > 0:
@@ -227,31 +221,31 @@ class EpisodeListModel(Gtk.ListStore):
         else:
             return None
 
-    def _filter_visible_func(self, model, iter, misc):
+    def _filter_visible_func(self, model, iterator, misc):
         # If searching is active, set visibility based on search text
         if self._search_term is not None and self._search_term != '':
-            episode = model.get_value(iter, self.C_EPISODE)
+            episode = model.get_value(iterator, self.C_EPISODE)
             if episode is None:
                 return False
 
             try:
                 return self._search_term_eql.match(episode)
-            except Exception as e:
+            except Exception:
                 return True
 
         if self._view_mode == self.VIEW_ALL:
             return True
         elif self._view_mode == self.VIEW_UNDELETED:
-            return model.get_value(iter, self.C_VIEW_SHOW_UNDELETED)
+            return model.get_value(iterator, self.C_VIEW_SHOW_UNDELETED)
         elif self._view_mode == self.VIEW_DOWNLOADED:
-            return model.get_value(iter, self.C_VIEW_SHOW_DOWNLOADED)
+            return model.get_value(iterator, self.C_VIEW_SHOW_DOWNLOADED)
         elif self._view_mode == self.VIEW_UNPLAYED:
-            return model.get_value(iter, self.C_VIEW_SHOW_UNPLAYED)
+            return model.get_value(iterator, self.C_VIEW_SHOW_UNPLAYED)
 
         return True
 
     def get_filtered_model(self):
-        """Returns a filtered version of this episode model
+        """Return a filtered version of this episode model.
 
         The filtered version should be displayed in the UI,
         as this model can have some filters set that should
@@ -260,7 +254,7 @@ class EpisodeListModel(Gtk.ListStore):
         return self._sorter
 
     def has_episodes(self):
-        """Returns True if episodes are visible (filtered)
+        """Return True if episodes are visible (filtered).
 
         If episodes are visible with the current filter
         applied, return True (otherwise return False).
@@ -268,17 +262,18 @@ class EpisodeListModel(Gtk.ListStore):
         return bool(len(self._filter))
 
     def set_view_mode(self, new_mode):
-        """Sets a new view mode for this model
+        """Set a new view mode for this model.
 
         After setting the view mode, the filtered model
-        might be updated to reflect the new mode."""
+        might be updated to reflect the new mode.
+        """
         if self._view_mode != new_mode:
             self._view_mode = new_mode
             self._filter.refilter()
             self._on_filter_changed(self.has_episodes())
 
     def get_view_mode(self):
-        """Returns the currently-set view mode"""
+        """Return the currently-set view mode."""
         return self._view_mode
 
     def set_search_term(self, new_term):
@@ -315,11 +310,10 @@ class EpisodeListModel(Gtk.ListStore):
         return ''.join(d)
 
     def replace_from_channel(self, channel):
-        """
-        Add episode from the given channel to this model.
+        """Add episode from the given channel to this model.
+
         Downloading should be a callback.
         """
-
         # Remove old episodes in the list store
         self.clear()
 
@@ -373,11 +367,11 @@ class EpisodeListModel(Gtk.ListStore):
             if row[self.C_URL] in urls:
                 self.update_by_iter(row.iter)
 
-    def update_by_filter_iter(self, iter):
+    def update_by_filter_iter(self, iterator):
         # Convenience function for use by "outside" methods that use iters
         # from the filtered episode list model (i.e. all UI things normally)
-        iter = self._sorter.convert_iter_to_child_iter(iter)
-        self.update_by_iter(self._filter.convert_iter_to_child_iter(iter))
+        iterator = self._sorter.convert_iter_to_child_iter(iterator)
+        self.update_by_iter(self._filter.convert_iter_to_child_iter(iterator))
 
     def get_update_fields(self, episode):
         tooltip = []
@@ -494,14 +488,15 @@ class EpisodeListModel(Gtk.ListStore):
                 self.C_FILESIZE_AND_TIME, episode.file_size,
         )
 
-    def update_by_iter(self, iter):
-        episode = self.get_value(iter, self.C_EPISODE)
+    def update_by_iter(self, iterator):
+        episode = self.get_value(iterator, self.C_EPISODE)
         if episode is not None:
-            self.set(iter, *self.get_update_fields(episode))
+            self.set(iterator, *self.get_update_fields(episode))
 
 
 class PodcastChannelProxy:
-    """ a bag of podcasts: 'All Episodes' or each section """
+    """A bag of podcasts: 'All Episodes' or each section."""
+
     def __init__(self, db, config, channels, section, model):
         self.ALL_EPISODES_PROXY = not bool(section)
         self._db = db
@@ -545,7 +540,7 @@ class PodcastChannelProxy:
             return total, deleted, new, downloaded, unplayed
 
     def get_all_episodes(self):
-        """Returns a generator that yields every episode"""
+        """Return a generator that yields every episode."""
         if self.model._search_term is not None:
             def matches(channel):
                 columns = (getattr(channel, c) for c in PodcastListModel.SEARCH_ATTRS)
@@ -572,8 +567,8 @@ class PodcastListModel(Gtk.ListStore):
     SEARCH_ATTRS = ('title', 'description', 'group_by')
 
     @classmethod
-    def row_separator_func(cls, model, iter):
-        return model.get_value(iter, cls.C_SEPARATOR)
+    def row_separator_func(cls, model, iterator):
+        return model.get_value(iterator, cls.C_SEPARATOR)
 
     def __init__(self, cover_downloader):
         Gtk.ListStore.__init__(self, str, str, str, GdkPixbuf.Pixbuf,
@@ -595,8 +590,8 @@ class PodcastListModel(Gtk.ListStore):
         self.ICON_DISABLED = 'media-playback-pause'
         self.ICON_ERROR = 'dialog-warning'
 
-    def _filter_visible_func(self, model, iter, misc):
-        channel = model.get_value(iter, self.C_CHANNEL)
+    def _filter_visible_func(self, model, iterator, misc):
+        channel = model.get_value(iterator, self.C_CHANNEL)
 
         # If searching is active, set visibility based on search text
         if self._search_term is not None and self._search_term != '':
@@ -605,7 +600,7 @@ class PodcastListModel(Gtk.ListStore):
                 if channel.ALL_EPISODES_PROXY:
                     return False
                 return any(key in getattr(ch, c).lower() for c in PodcastListModel.SEARCH_ATTRS for ch in channel.channels)
-            columns = (model.get_value(iter, c) for c in self.SEARCH_COLUMNS)
+            columns = (model.get_value(iterator, c) for c in self.SEARCH_COLUMNS)
             return any((key in c.lower() for c in columns if c is not None))
 
         # Show section if any of its channels have an update error
@@ -613,23 +608,23 @@ class PodcastListModel(Gtk.ListStore):
             if any(c._update_error is not None for c in channel.channels):
                 return True
 
-        if model.get_value(iter, self.C_SEPARATOR):
+        if model.get_value(iterator, self.C_SEPARATOR):
             return True
         elif getattr(channel, '_update_error', None) is not None:
             return True
         elif self._view_mode == EpisodeListModel.VIEW_ALL:
-            return model.get_value(iter, self.C_HAS_EPISODES)
+            return model.get_value(iterator, self.C_HAS_EPISODES)
         elif self._view_mode == EpisodeListModel.VIEW_UNDELETED:
-            return model.get_value(iter, self.C_VIEW_SHOW_UNDELETED)
+            return model.get_value(iterator, self.C_VIEW_SHOW_UNDELETED)
         elif self._view_mode == EpisodeListModel.VIEW_DOWNLOADED:
-            return model.get_value(iter, self.C_VIEW_SHOW_DOWNLOADED)
+            return model.get_value(iterator, self.C_VIEW_SHOW_DOWNLOADED)
         elif self._view_mode == EpisodeListModel.VIEW_UNPLAYED:
-            return model.get_value(iter, self.C_VIEW_SHOW_UNPLAYED)
+            return model.get_value(iterator, self.C_VIEW_SHOW_UNPLAYED)
 
         return True
 
     def get_filtered_model(self):
-        """Returns a filtered version of this episode model
+        """Return a filtered version of this episode model.
 
         The filtered version should be displayed in the UI,
         as this model can have some filters set that should
@@ -638,16 +633,17 @@ class PodcastListModel(Gtk.ListStore):
         return self._filter
 
     def set_view_mode(self, new_mode):
-        """Sets a new view mode for this model
+        """Set a new view mode for this model.
 
         After setting the view mode, the filtered model
-        might be updated to reflect the new mode."""
+        might be updated to reflect the new mode.
+        """
         if self._view_mode != new_mode:
             self._view_mode = new_mode
             self._filter.refilter()
 
     def get_view_mode(self):
-        """Returns the currently-set view mode"""
+        """Return the currently-set view mode."""
         return self._view_mode
 
     def set_search_term(self, new_term):
@@ -664,8 +660,8 @@ class PodcastListModel(Gtk.ListStore):
         self._cover_cache = {}
 
     def _resize_pixbuf_keep_ratio(self, url, pixbuf):
-        """
-        Resizes a GTK Pixbuf but keeps its aspect ratio.
+        """Resizes a GTK Pixbuf but keeps its aspect ratio.
+
         Returns None if the pixbuf does not need to be
         resized or the newly resized pixbuf if it does.
         """
@@ -728,7 +724,7 @@ class PodcastListModel(Gtk.ListStore):
                 logger.debug("cached thumb wrong size: %r != %i", (pixbuf.get_width(), pixbuf.get_height()), self._max_image_side)
                 return None
             return pixbuf
-        except Exception as e:
+        except Exception:
             logger.warning('Could not load cached cover art for %s', channel.url, exc_info=True)
             channel.cover_thumb = None
             channel.save()
@@ -745,11 +741,12 @@ class PodcastListModel(Gtk.ListStore):
         channel.save()
 
     def _get_cover_image(self, channel, add_overlay=False, pixbuf_overlay=None):
-        """ get channel's cover image. Callable from gtk thread.
-            :param channel: channel model
-            :param bool add_overlay: True to add a pause/error overlay
-            :param GdkPixbuf.Pixbux pixbuf_overlay: existing pixbuf if already loaded, as an optimization
-            :return GdkPixbuf.Pixbux: channel's cover image as pixbuf
+        """Get channel's cover image. Callable from gtk thread.
+
+        :param channel: channel model
+        :param bool add_overlay: True to add a pause/error overlay
+        :param GdkPixbuf.Pixbux pixbuf_overlay: existing pixbuf if already loaded, as an optimization
+        :return GdkPixbuf.Pixbux: channel's cover image as pixbuf
         """
         if self._cover_downloader is None:
             return pixbuf_overlay
@@ -844,8 +841,8 @@ class PodcastListModel(Gtk.ListStore):
 
         if config.ui.gtk.podcast_list.all_episodes and channels:
             all_episodes = PodcastChannelProxy(db, config, channels, '', self)
-            iter = self.append(channel_to_row(all_episodes))
-            self.update_by_iter(iter)
+            iterator = self.append(channel_to_row(all_episodes))
+            self.update_by_iter(iterator)
 
             # Separator item
             if not config.ui.gtk.podcast_list.sections:
@@ -867,11 +864,11 @@ class PodcastListModel(Gtk.ListStore):
             if config.ui.gtk.podcast_list.sections and section is not None:
                 section_channels = list(section_channels)
                 section_obj = PodcastChannelProxy(db, config, section_channels, section, self)
-                iter = self.append(section_to_row(section_obj))
-                self.update_by_iter(iter)
+                iterator = self.append(section_to_row(section_obj))
+                self.update_by_iter(iterator)
             for channel in section_channels:
-                iter = self.append(channel_to_row(channel, True))
-                self.update_by_iter(iter)
+                iterator = self.append(channel_to_row(channel, True))
+                self.update_by_iter(iterator)
 
     def get_filter_path_from_url(self, url):
         # Return the path of the filtered model for a given URL
@@ -901,13 +898,13 @@ class PodcastListModel(Gtk.ListStore):
             if row[self.C_URL] in urls:
                 self.update_by_iter(row.iter)
 
-    def iter_is_first_row(self, iter):
-        iter = self._filter.convert_iter_to_child_iter(iter)
-        path = self.get_path(iter)
+    def iter_is_first_row(self, iterator):
+        iterator = self._filter.convert_iter_to_child_iter(iterator)
+        path = self.get_path(iterator)
         return (path == Gtk.TreePath.new_first())
 
-    def update_by_filter_iter(self, iter):
-        self.update_by_iter(self._filter.convert_iter_to_child_iter(iter))
+    def update_by_filter_iter(self, iterator):
+        self.update_by_iter(self._filter.convert_iter_to_child_iter(iterator))
 
     def update_all(self):
         for row in self:
@@ -918,12 +915,12 @@ class PodcastListModel(Gtk.ListStore):
             if isinstance(row[self.C_CHANNEL], PodcastChannelProxy) and not row[self.C_CHANNEL].ALL_EPISODES_PROXY:
                 self.update_by_iter(row.iter)
 
-    def update_by_iter(self, iter):
-        if iter is None:
+    def update_by_iter(self, iterator):
+        if iterator is None:
             return
 
         # Given a GtkTreeIter, update volatile information
-        channel = self.get_value(iter, self.C_CHANNEL)
+        channel = self.get_value(iterator, self.C_CHANNEL)
 
         if channel is SeparatorMarker:
             return
@@ -946,7 +943,7 @@ class PodcastListModel(Gtk.ListStore):
             pill_image = self._get_pill_image(channel, downloaded, unplayed)
             cover_image = self._get_cover_image(channel, True)
 
-        self.set(iter,
+        self.set(iterator,
                 self.C_TITLE, channel.title,
                 self.C_DESCRIPTION, description,
                 self.C_COVER, cover_image,
