@@ -16,9 +16,9 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
+import datetime
 import html
 import logging
-import re
 from urllib.parse import urlparse
 
 import gpodder
@@ -31,7 +31,7 @@ from gpodder.gtkui.draw import (draw_text_box_centered, get_background_color,
 import gi  # isort:skip
 gi.require_version('Gdk', '3.0')  # isort:skip
 gi.require_version('Gtk', '3.0')  # isort:skip
-from gi.repository import Gdk, Gtk, Pango  # isort:skip
+from gi.repository import Gdk, Gio, GLib, Gtk, Pango  # isort:skip
 
 
 _ = gpodder.gettext
@@ -57,7 +57,7 @@ def get_shownotes(enable_html, pane):
 class gPodderShownotes:
     def __init__(self, shownotes_pane):
         self.shownotes_pane = shownotes_pane
-        self.details_fmt = _('%s | %s | %s')
+        self.details_fmt = _('%(date)s | %(size)s | %(duration)s')
 
         self.scrolled_window = Gtk.ScrolledWindow()
         self.scrolled_window.set_shadow_type(Gtk.ShadowType.IN)
@@ -193,26 +193,34 @@ class gPodderShownotesText(gPodderShownotes):
         self.text_view.connect('button-release-event', self.on_button_release)
         self.text_view.connect('key-press-event', self.on_key_press)
         self.text_view.connect('motion-notify-event', self.on_hover_hyperlink)
+        self.populate_popup_id = None
         return self.text_view
 
     def update(self, episode):
+        self.scrolled_window.get_vadjustment().set_value(0)
+
         heading = episode.title
         subheading = _('from %s') % (episode.channel.title)
-        details = self.details_fmt % (
-            util.format_date(episode.published),
-            util.format_filesize(episode.file_size, digits=1)
+        details = self.details_fmt % {
+            'date': '{} {}'.format(datetime.datetime.fromtimestamp(episode.published).strftime('%H:%M'),
+                util.format_date(episode.published)),
+            'size': util.format_filesize(episode.file_size, digits=1)
             if episode.file_size > 0 else "-",
-            episode.get_play_info_string())
+            'duration': episode.get_play_info_string()}
         self.define_colors()
         hyperlinks = [(0, None)]
         self.text_buffer.set_text('')
+        if episode.link:
+            hyperlinks.append((self.text_buffer.get_char_count(), episode.link))
         self.text_buffer.insert_with_tags_by_name(self.text_buffer.get_end_iter(), heading, 'heading')
+        if episode.link:
+            hyperlinks.append((self.text_buffer.get_char_count(), None))
         self.text_buffer.insert_at_cursor('\n')
         self.text_buffer.insert_with_tags_by_name(self.text_buffer.get_end_iter(), subheading, 'subheading')
         self.text_buffer.insert_at_cursor('\n')
         self.text_buffer.insert_with_tags_by_name(self.text_buffer.get_end_iter(), details, 'details')
         self.text_buffer.insert_at_cursor('\n\n')
-        for target, text in util.extract_hyperlinked_text(episode.description_html or episode.description):
+        for target, text in util.extract_hyperlinked_text(episode.html_description()):
             hyperlinks.append((self.text_buffer.get_char_count(), target))
             if target:
                 self.text_buffer.insert_with_tags_by_name(
@@ -224,6 +232,34 @@ class gPodderShownotesText(gPodderShownotes):
         self.hyperlinks = [(start, end, url) for (start, url), (end, _) in zip(hyperlinks, hyperlinks[1:]) if url]
         self.text_buffer.place_cursor(self.text_buffer.get_start_iter())
 
+        if self.populate_popup_id is not None:
+            self.text_view.disconnect(self.populate_popup_id)
+        self.populate_popup_id = self.text_view.connect('populate-popup', self.on_populate_popup)
+        self.episode = episode
+
+    def on_populate_popup(self, textview, context_menu):
+        # TODO: Remove items from context menu that are always insensitive in a read-only buffer
+
+        if self.episode.link:
+            # TODO: It is currently not possible to copy links in description.
+            # Detect if context menu was opened on a hyperlink and add
+            # "Open Link" and "Copy Link Address" menu items.
+            # See https://github.com/gpodder/gpodder/issues/1097
+
+            item = Gtk.SeparatorMenuItem()
+            item.show()
+            context_menu.append(item)
+            # label links can be opened from context menu or by clicking them, do the same here
+            item = Gtk.MenuItem(label=_('Open Episode Title Link'))
+            item.connect('activate', lambda i: util.open_website(self.episode.link))
+            item.show()
+            context_menu.append(item)
+            # hack to allow copying episode.link
+            item = Gtk.MenuItem(label=_('Copy Episode Title Link Address'))
+            item.connect('activate', lambda i: util.copy_text_to_clipboard(self.episode.link))
+            item.show()
+            context_menu.append(item)
+
     def on_button_release(self, widget, event):
         if event.button == 1:
             self.activate_links()
@@ -236,7 +272,8 @@ class gPodderShownotesText(gPodderShownotes):
         return False
 
     def hyperlink_at_pos(self, pos):
-        """
+        """Return the hyperlink in the given position, or None.
+
         :param int pos: offset in text buffer
         :return str: hyperlink target at pos if any or None
         """
@@ -287,6 +324,8 @@ class gPodderShownotesHTML(gPodderShownotes):
         return self.html_view
 
     def update(self, episode):
+        self.scrolled_window.get_vadjustment().set_value(0)
+
         self.define_colors()
 
         if episode.has_website_link():
@@ -305,19 +344,18 @@ class gPodderShownotesHTML(gPodderShownotes):
             self.manager.add_style_sheet(stylesheet)
         heading = '<h3>%s</h3>' % html.escape(episode.title)
         subheading = _('from %s') % html.escape(episode.channel.title)
-        details = '<small>%s</small>' % html.escape(self.details_fmt % (
-            util.format_date(episode.published),
-            util.format_filesize(episode.file_size, digits=1)
+        details = '<small>%s</small>' % html.escape(self.details_fmt % {
+            'date': '{} {}'.format(datetime.datetime.fromtimestamp(episode.published).strftime('%H:%M'),
+                util.format_date(episode.published)),
+            'size': util.format_filesize(episode.file_size, digits=1)
             if episode.file_size > 0 else "-",
-            episode.get_play_info_string()))
-        header_html = _('<div id="gpodder-title">\n%s\n<p>%s</p>\n<p>%s</p></div>\n') % (heading, subheading, details)
-        description_html = episode.description_html
-        if not description_html:
-            description_html = re.sub(r'\n', '<br>\n', episode.description)
+            'duration': episode.get_play_info_string()})
+        header_html = _('<div id="gpodder-title">\n%(heading)s\n<p>%(subheading)s</p>\n<p>%(details)s</p></div>\n') \
+            % {'heading': heading, 'subheading': subheading, 'details': details}
         # uncomment to prevent background override in html shownotes
         # self.manager.remove_all_style_sheets ()
         logger.debug("base uri: %s (chan:%s)", self._base_uri, episode.channel.url)
-        self.html_view.load_html(header_html + description_html, self._base_uri)
+        self.html_view.load_html(header_html + episode.html_description(), self._base_uri)
         # uncomment to show web inspector
         # self.html_view.get_inspector().show()
         self.episode = episode
@@ -373,15 +411,16 @@ class gPodderShownotesHTML(gPodderShownotes):
             if req.get_uri() in (self._base_uri, 'about:blank'):
                 decision.use()
             else:
-                logger.debug("refusing to go to %s (base URI=%s)", req.get_uri(), self._base_uri)
+                # Avoid opening the page inside the WebView and open in the browser instead
                 decision.ignore()
+                util.open_website(req.get_uri())
             return False
         else:
             decision.use()
             return False
 
-    def on_open_in_browser(self, action):
-        util.open_website(action.url)
+    def on_open_in_browser(self, action, var):
+        util.open_website(var.get_string())
 
     def on_authenticate(self, view, request):
         if request.is_retry():
@@ -411,10 +450,10 @@ class gPodderShownotesHTML(gPodderShownotes):
             return False
 
     def create_open_item(self, name, label, url):
-        action = Gtk.Action.new(name, label, None, Gtk.STOCK_OPEN)
-        action.url = url
+        action = Gio.SimpleAction.new(name, GLib.VariantType.new('s'))
         action.connect('activate', self.on_open_in_browser)
-        return WebKit2.ContextMenuItem.new(action)
+        var = GLib.Variant.new_string(url)
+        return WebKit2.ContextMenuItem.new_from_gaction(action, label, var)
 
     def get_stylesheet(self):
         if self.stylesheet is None:
