@@ -1,8 +1,7 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2018 The gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,54 +23,62 @@
 #  Thomas Perl <thp@gpodder.org>; 2010-01-19
 #
 
-import gpodder
-_ = gpodder.gettext
-
 import atexit
-import datetime
 import calendar
+import datetime
+import logging
 import os
 import sys
-import threading
 import time
 
-import logging
-logger = logging.getLogger(__name__)
-
-from gpodder import util
-from gpodder import minidb
-
 # Append gPodder's user agent to mygpoclient's user agent
 import mygpoclient
+from mygpoclient import api, public
+from mygpoclient import util as mygpoutil
+
+import gpodder
+from gpodder import minidb, util
+
+_ = gpodder.gettext
+
+
+logger = logging.getLogger(__name__)
+
+
 mygpoclient.user_agent += ' ' + gpodder.user_agent
 
+# 2013-02-08: We should update this to 1.7 once we use the new features
 MYGPOCLIENT_REQUIRED = '1.4'
 
-if not hasattr(mygpoclient, 'require_version') or \
-        not mygpoclient.require_version(MYGPOCLIENT_REQUIRED):
-    print >>sys.stderr, """
+if (not hasattr(mygpoclient, 'require_version')
+        or not mygpoclient.require_version(MYGPOCLIENT_REQUIRED)):
+    print("""
     Please upgrade your mygpoclient library.
     See http://thp.io/2010/mygpoclient/
 
     Required version:  %s
     Installed version: %s
-    """ % (MYGPOCLIENT_REQUIRED, mygpoclient.__version__)
+    """ % (MYGPOCLIENT_REQUIRED, mygpoclient.__version__), file=sys.stderr)
     sys.exit(1)
 
-from mygpoclient import api
-from mygpoclient import public
+try:
+    from mygpoclient.simple import MissingCredentials
 
-from mygpoclient import util as mygpoutil
+except ImportError:
+    # if MissingCredentials does not yet exist in the installed version of
+    # mygpoclient, we use an object that can never be raised/caught
+    MissingCredentials = object()
+
+
+EPISODE_ACTIONS_BATCH_SIZE = 100
 
-EXAMPLES_OPML = 'http://gpodder.org/directory.opml'
-TOPLIST_OPML = 'http://gpodder.org/toplist.opml'
 
 # Database model classes
 class SinceValue(object):
     __slots__ = {'host': str, 'device_id': str, 'category': int, 'since': int}
 
     # Possible values for the "category" field
-    PODCASTS, EPISODES = range(2)
+    PODCASTS, EPISODES = list(range(2))
 
     def __init__(self, host, device_id, category, since=0):
         self.host = host
@@ -79,11 +86,12 @@ class SinceValue(object):
         self.category = category
         self.since = since
 
+
 class SubscribeAction(object):
     __slots__ = {'action_type': int, 'url': str}
 
     # Possible values for the "action_type" field
-    ADD, REMOVE = range(2)
+    ADD, REMOVE = list(range(2))
 
     def __init__(self, action_type, url):
         self.action_type = action_type
@@ -114,8 +122,11 @@ class SubscribeAction(object):
 
         raise ValueError('Cannot undo action: %r' % action)
 
+
 # New entity name for "received" actions
-class ReceivedSubscribeAction(SubscribeAction): pass
+class ReceivedSubscribeAction(SubscribeAction):
+    pass
+
 
 class UpdateDeviceAction(object):
     __slots__ = {'device_id': str, 'caption': str, 'device_type': str}
@@ -125,12 +136,13 @@ class UpdateDeviceAction(object):
         self.caption = caption
         self.device_type = device_type
 
+
 class EpisodeAction(object):
     __slots__ = {'podcast_url': str, 'episode_url': str, 'device_id': str,
                  'action': str, 'timestamp': int,
                  'started': int, 'position': int, 'total': int}
 
-    def __init__(self, podcast_url, episode_url, device_id, \
+    def __init__(self, podcast_url, episode_url, device_id,
             action, timestamp, started, position, total):
         self.podcast_url = podcast_url
         self.episode_url = episode_url
@@ -141,8 +153,11 @@ class EpisodeAction(object):
         self.position = position
         self.total = total
 
+
 # New entity name for "received" actions
-class ReceivedEpisodeAction(EpisodeAction): pass
+class ReceivedEpisodeAction(EpisodeAction):
+    pass
+
 
 class RewrittenUrl(object):
     __slots__ = {'old_url': str, 'new_url': str}
@@ -153,7 +168,6 @@ class RewrittenUrl(object):
 # End Database model classes
 
 
-
 # Helper class for displaying changes in the UI
 class Change(object):
     def __init__(self, action, podcast=None):
@@ -189,7 +203,7 @@ class MygPoClient(object):
         atexit.register(self._at_exit)
 
     def create_device(self):
-        """Uploads the device changes to the server
+        """Upload the device changes to the server.
 
         This should be called when device settings change
         or when the mygpo client functionality is enabled.
@@ -198,13 +212,13 @@ class MygPoClient(object):
         self._store.remove(self._store.load(UpdateDeviceAction))
 
         # Insert our new update action
-        action = UpdateDeviceAction(self.device_id, \
-                self._config.mygpo.device.caption, \
+        action = UpdateDeviceAction(self.device_id,
+                self._config.mygpo.device.caption,
                 self._config.mygpo.device.type)
         self._store.save(action)
 
     def get_rewritten_urls(self):
-        """Returns a list of rewritten URLs for uploads
+        """Return a list of rewritten URLs for uploads.
 
         This should be called regularly. Every object returned
         should be merged into the database, and the old_url
@@ -215,7 +229,7 @@ class MygPoClient(object):
         return rewritten_urls
 
     def process_episode_actions(self, find_episode, on_updated=None):
-        """Process received episode actions
+        """Process received episode actions.
 
         The parameter "find_episode" should be a function accepting
         two parameters (podcast_url and episode_url). It will be used
@@ -242,8 +256,8 @@ class MygPoClient(object):
                 logger.debug('Play action for %s', episode.url)
                 episode.mark(is_played=True)
 
-                if (action.timestamp > episode.current_position_updated and
-                        action.position is not None):
+                if (action.timestamp > episode.current_position_updated
+                        and action.position is not None):
                     logger.debug('Updating position for %s', episode.url)
                     episode.current_position = action.position
                     episode.current_position_updated = action.timestamp
@@ -270,11 +284,11 @@ class MygPoClient(object):
         logger.debug('Received episode actions processed.')
 
     def get_received_actions(self):
-        """Returns a list of ReceivedSubscribeAction objects
+        """Return a list of ReceivedSubscribeAction objects.
 
         The list might be empty. All these actions have to
         be processed. The user should confirm which of these
-        actions should be taken, the reest should be rejected.
+        actions should be taken, the rest should be rejected.
 
         Use confirm_received_actions and reject_received_actions
         to return and finalize the actions received by this
@@ -283,7 +297,7 @@ class MygPoClient(object):
         return self._store.load(ReceivedSubscribeAction)
 
     def confirm_received_actions(self, actions):
-        """Confirm that a list of actions has been processed
+        """Confirm that a list of actions has been processed.
 
         The UI should call this with a list of actions that
         have been accepted by the user and processed by the
@@ -293,7 +307,7 @@ class MygPoClient(object):
         self._store.remove(actions)
 
     def reject_received_actions(self, actions):
-        """Reject (undo) a list of ReceivedSubscribeAction objects
+        """Reject (undo) a list of ReceivedSubscribeAction objects.
 
         The UI should call this with a list of actions that
         have been rejected by the user. A reversed set of
@@ -316,7 +330,9 @@ class MygPoClient(object):
         return self._config.mygpo.device.uid
 
     def can_access_webservice(self):
-        return self._config.mygpo.enabled and self._config.mygpo.device.uid
+        return self._config.mygpo.enabled and \
+               self._config.mygpo.username and \
+               self._config.mygpo.device.uid
 
     def set_subscriptions(self, urls):
         if self.can_access_webservice():
@@ -327,13 +343,13 @@ class MygPoClient(object):
             raise Exception('Webservice access not enabled')
 
     def _convert_played_episode(self, episode, start, end, total):
-        return EpisodeAction(episode.channel.url, \
-                episode.url, self.device_id, 'play', \
+        return EpisodeAction(episode.channel.url,
+                episode.url, self.device_id, 'play',
                 int(time.time()), start, end, total)
 
     def _convert_episode(self, episode, action):
-        return EpisodeAction(episode.channel.url, \
-                episode.url, self.device_id, action, \
+        return EpisodeAction(episode.channel.url,
+                episode.url, self.device_id, action,
                 int(time.time()), None, None, None)
 
     def on_delete(self, episodes):
@@ -416,8 +432,9 @@ class MygPoClient(object):
                 else:
                     must_retry = True
 
-                if not must_retry:
-                    # No more pending actions. Ready to quit.
+                if not must_retry or not self.can_access_webservice():
+                    # No more pending actions, or no longer enabled.
+                    # Ready to quit.
                     break
 
             logger.debug('Worker thread finished.')
@@ -429,7 +446,7 @@ class MygPoClient(object):
 
     def flush(self, now=False):
         if not self.can_access_webservice():
-            logger.warn('Flush requested, but sync disabled.')
+            logger.debug('Flush requested, but sync disabled.')
             return
 
         if self._worker_thread is None or now:
@@ -437,9 +454,7 @@ class MygPoClient(object):
                 logger.debug('Flushing NOW.')
             else:
                 logger.debug('Flush requested.')
-            self._worker_thread = threading.Thread(target=self._worker_proc, args=[now])
-            self._worker_thread.setDaemon(True)
-            self._worker_thread.start()
+            self._worker_thread = util.run_in_background(lambda: self._worker_proc(now), True)
         else:
             logger.debug('Flush requested, already waiting.')
 
@@ -459,24 +474,24 @@ class MygPoClient(object):
         def convert_to_api(action):
             dt = datetime.datetime.utcfromtimestamp(action.timestamp)
             action_ts = mygpoutil.datetime_to_iso8601(dt)
-            return api.EpisodeAction(action.podcast_url, \
-                    action.episode_url, action.action, \
-                    action.device_id, action_ts, \
+            return api.EpisodeAction(action.podcast_url,
+                    action.episode_url, action.action,
+                    action.device_id, action_ts,
                     action.started, action.position, action.total)
 
         def convert_from_api(action):
             dt = mygpoutil.iso8601_to_datetime(action.timestamp)
             action_ts = calendar.timegm(dt.timetuple())
-            return ReceivedEpisodeAction(action.podcast, \
-                    action.episode, action.device, \
-                    action.action, action_ts, \
+            return ReceivedEpisodeAction(action.podcast,
+                    action.episode, action.device,
+                    action.action, action_ts,
                     action.started, action.position, action.total)
 
         try:
             # Load the "since" value from the database
-            since_o = self._store.get(SinceValue, host=self.host, \
-                                                  device_id=self.device_id, \
-                                                  category=SinceValue.EPISODES)
+            since_o = self._store.get(SinceValue, host=self.host,
+                                      device_id=self.device_id,
+                                      category=SinceValue.EPISODES)
 
             # Use a default since object for the first-time case
             if since_o is None:
@@ -492,22 +507,39 @@ class MygPoClient(object):
 
                 # Save the "since" value for later use
                 self._store.update(since_o, since=changes.since)
-            except Exception, e:
-                logger.warn('Exception while polling for episodes.', exc_info=True)
+
+            except (MissingCredentials, mygpoclient.http.Unauthorized):
+                # handle outside
+                raise
+
+            except Exception:
+                logger.warning('Exception while polling for episodes.', exc_info=True)
 
             # Step 2: Upload Episode actions
 
-            # Convert actions to the mygpoclient format for uploading
-            episode_actions = [convert_to_api(a) for a in actions]
+            # Uploads are done in batches; uploading can resume if only parts
+            # be uploaded; avoids empty uploads as well
+            for lower in range(0, len(actions), EPISODE_ACTIONS_BATCH_SIZE):
+                batch = actions[lower:(lower + EPISODE_ACTIONS_BATCH_SIZE)]
 
-            # Upload the episode actions
-            self._client.upload_episode_actions(episode_actions)
+                # Convert actions to the mygpoclient format for uploading
+                episode_actions = [convert_to_api(a) for a in batch]
+
+                # Upload the episode actions
+                self._client.upload_episode_actions(episode_actions)
+
+                # Actions have been uploaded to the server - remove them
+                self._store.remove(batch)
 
-            # Actions have been uploaded to the server - remove them
-            self._store.remove(actions)
             logger.debug('Episode actions have been uploaded to the server.')
             return True
-        except Exception, e:
+
+        except (MissingCredentials, mygpoclient.http.Unauthorized):
+            logger.warning('Invalid credentials. Disabling gpodder.net.')
+            self._config.mygpo.enabled = False
+            return False
+
+        except Exception as e:
             logger.error('Cannot upload episode actions: %s', str(e), exc_info=True)
             return False
 
@@ -515,9 +547,9 @@ class MygPoClient(object):
         logger.debug('Starting subscription sync.')
         try:
             # Load the "since" value from the database
-            since_o = self._store.get(SinceValue, host=self.host, \
-                                                  device_id=self.device_id, \
-                                                  category=SinceValue.PODCASTS)
+            since_o = self._store.get(SinceValue, host=self.host,
+                                      device_id=self.device_id,
+                                      category=SinceValue.PODCASTS)
 
             # Use a default since object for the first-time case
             if since_o is None:
@@ -566,43 +598,70 @@ class MygPoClient(object):
             self._store.remove(actions)
             logger.debug('All actions have been uploaded to the server.')
             return True
-        except Exception, e:
+
+        except (MissingCredentials, mygpoclient.http.Unauthorized):
+            logger.warning('Invalid credentials. Disabling gpodder.net.')
+            self._config.mygpo.enabled = False
+            return False
+
+        except Exception as e:
             logger.error('Cannot upload subscriptions: %s', str(e), exc_info=True)
             return False
 
     def update_device(self, action):
         try:
             logger.debug('Uploading device settings...')
-            self._client.update_device_settings(action.device_id, \
+            self._client.update_device_settings(action.device_id,
                     action.caption, action.device_type)
             logger.debug('Device settings uploaded.')
             return True
-        except Exception, e:
+
+        except (MissingCredentials, mygpoclient.http.Unauthorized):
+            logger.warning('Invalid credentials. Disabling gpodder.net.')
+            self._config.mygpo.enabled = False
+            return False
+
+        except Exception as e:
             logger.error('Cannot update device %s: %s', self.device_id,
                 str(e), exc_info=True)
             return False
 
     def get_devices(self):
         result = []
-        for d in self._client.get_devices():
+
+        try:
+            devices = self._client.get_devices()
+
+        except (MissingCredentials, mygpoclient.http.Unauthorized):
+            logger.warning('Invalid credentials. Disabling gpodder.net.')
+            self._config.mygpo.enabled = False
+            raise
+
+        for d in devices:
             result.append((d.device_id, d.caption, d.type))
         return result
 
     def open_website(self):
         util.open_website('http://' + self._config.mygpo.server)
 
+    def get_download_user_subscriptions_url(self):
+        OPML_URL = self._client.locator.subscriptions_uri()
+        url = util.url_add_authentication(OPML_URL,
+                self._config.mygpo.username,
+                self._config.mygpo.password)
+        return url
+
 
 class Directory(object):
     def __init__(self):
         self.client = public.PublicClient()
 
     def toplist(self):
-        return [(p.title or p.url, p.url)
+        return [(p.title or p.url, p.url, None)
                 for p in self.client.get_toplist()
                 if p.url]
 
     def search(self, query):
-        return [(p.title or p.url, p.url)
+        return [(p.title or p.url, p.url, None)
                 for p in self.client.search_podcasts(query)
                 if p.url]
-
