@@ -37,10 +37,10 @@ class Player(object):
         self.gpodder = None
 
     def is_installed(self):
-        raise NotImplemented('Must be implemented by subclass')
+        raise NotImplementedError('Must be implemented by subclass')
 
     def open_files(self, filenames):
-        raise NotImplemented('Must be implemented by subclass')
+        raise NotImplementedError('Must be implemented by subclass')
 
     def enqueue_episodes(self, episodes, config=None):
         filenames = [episode.get_playback_url(config=config) for episode in episodes]
@@ -81,9 +81,8 @@ class Win32Player(Player):
 
 
 class MPRISResumer(FreeDesktopPlayer):
-    """
-    resume episod playback at saved time
-    """
+    """Resume episode playback at saved time."""
+
     OBJECT_PLAYER = '/org/mpris/MediaPlayer2'
     OBJECT_DBUS = '/org/freedesktop/DBus'
     INTERFACE_PLAYER = 'org.mpris.MediaPlayer2.Player'
