@@ -15,7 +15,7 @@ _ = gpodder.gettext
 __title__ = _('Search for new episodes on startup')
 __description__ = _('Starts the search for new episodes on startup')
 __authors__ = 'Bernd Schlapsi <brot@gmx.info>'
-__doc__ = 'http://wiki.gpodder.org/wiki/Extensions/SearchEpisodeOnStartup'
+__doc__ = 'https://gpodder.github.io/docs/extensions/searchepisodeonstartup.html'
 __payment__ = 'https://flattr.com/submit/auto?user_id=BerndSch&url=http://wiki.gpodder.org/wiki/Extensions/SearchEpisodeOnStartup'
 __category__ = 'interface'
 __only_for__ = 'gtk'
@@ -31,6 +31,6 @@ class gPodderExtension:
         if name == 'gpodder-gtk':
             self.gpodder = ui_object
 
-    def on_application_started(self):
+    def on_find_partial_downloads_done(self):
         if self.gpodder:
             self.gpodder.update_feed_cache()
